@@ -46,14 +46,18 @@ class IMAPTest < Test::Unit::TestCase
       # Otherwise, failures can't logout and need to wait for the timeout.
       verified, imap = :unknown, nil
       assert_nothing_raised do
-        imaps_test do |port|
-          imap = Net::IMAP.new("localhost",
-                               port: port,
-                               ssl: { :ca_file => CA_FILE })
-          verified = imap.tls_verified?
-          imap
-        rescue SystemCallError
-          skip $!
+        begin
+          imaps_test do |port|
+            imap = Net::IMAP.new("localhost",
+                                port: port,
+                                ssl: { :ca_file => CA_FILE })
+            verified = imap.tls_verified?
+            imap
+          rescue SystemCallError
+            skip $!
+          end
+        rescue OpenSSL::SSL::SSLError => e
+          raise e unless /darwin/ =~ RUBY_PLATFORM
         end
       end
       assert_equal true, verified
@@ -69,14 +73,18 @@ class IMAPTest < Test::Unit::TestCase
       # Otherwise, failures can't logout and need to wait for the timeout.
       verified, imap = :unknown, nil
       assert_nothing_raised do
-        imaps_test do |port|
-          imap = Net::IMAP.new(
-            server_addr,
-            port: port,
-            ssl: { :verify_mode => OpenSSL::SSL::VERIFY_NONE }
-          )
-          verified = imap.tls_verified?
-          imap
+        begin
+          imaps_test do |port|
+            imap = Net::IMAP.new(
+              server_addr,
+              port: port,
+              ssl: { :verify_mode => OpenSSL::SSL::VERIFY_NONE }
+            )
+            verified = imap.tls_verified?
+            imap
+          end
+        rescue OpenSSL::SSL::SSLError => e
+          raise e unless /darwin/ =~ RUBY_PLATFORM
         end
       end
       assert_equal false, verified
@@ -102,6 +110,8 @@ class IMAPTest < Test::Unit::TestCase
 
   if defined?(OpenSSL::SSL)
     def test_starttls_unknown_ca
+      omit "This test is not working with Windows" if RUBY_PLATFORM =~ /mswin|mingw/
+
       imap = nil
       ex = nil
       starttls_test do |port|
@@ -625,17 +635,20 @@ class IMAPTest < Test::Unit::TestCase
         imap.__send__(:send_command, "TEST", 2**32)
       end
       # MessageSet numbers may be non-zero uint32
-      assert_raise(Net::IMAP::DataFormatError) do
-        imap.__send__(:send_command, "TEST", Net::IMAP::MessageSet.new(-1))
-      end
-      assert_raise(Net::IMAP::DataFormatError) do
-        imap.__send__(:send_command, "TEST", Net::IMAP::MessageSet.new(0))
-      end
-      imap.__send__(:send_command, "TEST", Net::IMAP::MessageSet.new(1))
-      imap.__send__(:send_command, "TEST", Net::IMAP::MessageSet.new(2**32 - 1))
-      assert_raise(Net::IMAP::DataFormatError) do
-        imap.__send__(:send_command, "TEST", Net::IMAP::MessageSet.new(2**32))
+      stderr = EnvUtil.verbose_warning do
+        assert_raise(Net::IMAP::DataFormatError) do
+          imap.__send__(:send_command, "TEST", Net::IMAP::MessageSet.new(-1))
+        end
+        assert_raise(Net::IMAP::DataFormatError) do
+          imap.__send__(:send_command, "TEST", Net::IMAP::MessageSet.new(0))
+        end
+        imap.__send__(:send_command, "TEST", Net::IMAP::MessageSet.new(1))
+        imap.__send__(:send_command, "TEST", Net::IMAP::MessageSet.new(2**32 - 1))
+        assert_raise(Net::IMAP::DataFormatError) do
+          imap.__send__(:send_command, "TEST", Net::IMAP::MessageSet.new(2**32))
+        end
       end
+      assert_match(/DEPRECATED:.+MessageSet.+replace.+with.+SequenceSet/, stderr)
       # SequenceSet numbers may be non-zero uint3, and -1 is translated to *
       imap.__send__(:send_command, "TEST", Net::IMAP::SequenceSet.new(-1))
       assert_raise(Net::IMAP::DataFormatError) do
@@ -652,6 +665,56 @@ class IMAPTest < Test::Unit::TestCase
     end
   end
 
+  test("send PartialRange args") do
+    with_fake_server do |server, imap|
+      server.on "TEST", &:done_ok
+      send_partial_ranges = ->(*args) do
+        args.map! { Net::IMAP::PartialRange[_1] }
+        imap.__send__(:send_command, "TEST", *args)
+      end
+      # simple strings
+      send_partial_ranges.call "1:5", "-5:-1"
+      assert_equal "1:5 -5:-1", server.commands.pop.args
+      # backwards strings are reversed
+      send_partial_ranges.call "5:1", "-1:-5"
+      assert_equal "1:5 -5:-1", server.commands.pop.args
+      # simple ranges
+      send_partial_ranges.call 1..5, -5..-1
+      assert_equal "1:5 -5:-1", server.commands.pop.args
+      # exclusive ranges drop end
+      send_partial_ranges.call 1...5, -5...-1
+      assert_equal "1:4 -5:-2", server.commands.pop.args
+
+      # backwards ranges are invalid
+      assert_raise(ArgumentError) do send_partial_ranges.call( 5.. 1) end
+      assert_raise(ArgumentError) do send_partial_ranges.call(-1..-5) end
+
+      # bounds checks
+      uint32_max = 2**32 - 1
+      not_uint32 = 2**32
+      send_partial_ranges.call 500..uint32_max
+      assert_equal "500:#{uint32_max}", server.commands.pop.args
+      send_partial_ranges.call 500...not_uint32
+      assert_equal "500:#{uint32_max}", server.commands.pop.args
+      send_partial_ranges.call "#{uint32_max}:500"
+      assert_equal "500:#{uint32_max}", server.commands.pop.args
+
+      send_partial_ranges.call(-uint32_max..-500)
+      assert_equal "-#{uint32_max}:-500", server.commands.pop.args
+      send_partial_ranges.call "-500:-#{uint32_max}"
+      assert_equal "-#{uint32_max}:-500", server.commands.pop.args
+
+      assert_raise(ArgumentError) do send_partial_ranges.call("foo") end
+      assert_raise(ArgumentError) do send_partial_ranges.call("foo:bar") end
+      assert_raise(ArgumentError) do send_partial_ranges.call("1.2:3.5") end
+      assert_raise(ArgumentError) do send_partial_ranges.call("1:*") end
+      assert_raise(ArgumentError) do send_partial_ranges.call("1:#{not_uint32}") end
+      assert_raise(ArgumentError) do send_partial_ranges.call(1..) end
+      assert_raise(ArgumentError) do send_partial_ranges.call(1..not_uint32) end
+      assert_raise(ArgumentError) do send_partial_ranges.call(..1) end
+    end
+  end
+
   def test_send_literal
     server = create_tcp_server
     port = server.addr[1]
@@ -1010,7 +1073,7 @@ EOF
     end
   end
 
-  def test_uidplus_uid_expunge
+  test "#uid_expunge with EXPUNGE responses" do
     with_fake_server(select: "INBOX",
                      extensions: %i[UIDPLUS]) do |server, imap|
       server.on "UID EXPUNGE" do |resp|
@@ -1026,6 +1089,24 @@ EOF
     end
   end
 
+  test "#uid_expunge with VANISHED response" do
+    with_fake_server(select: "INBOX",
+                     extensions: %i[UIDPLUS]) do |server, imap|
+      server.on "UID EXPUNGE" do |resp|
+        resp.untagged("VANISHED 1001,1003")
+        resp.done_ok
+      end
+      response = imap.uid_expunge(1000..1003)
+      cmd = server.commands.pop
+      assert_equal ["UID EXPUNGE", "1000:1003"], [cmd.name, cmd.args]
+      assert_equal(
+        Net::IMAP::VanishedData[uids: [1001, 1003], earlier: false],
+        response
+      )
+      assert_equal([], imap.clear_responses("VANISHED"))
+    end
+  end
+
   def test_uidplus_appenduid
     with_fake_server(select: "INBOX",
                      extensions: %i[UIDPLUS]) do |server, imap|
@@ -1122,10 +1203,62 @@ EOF
     end
   end
 
+  test "#fetch with FETCH responses" do
+    with_fake_server select: "inbox" do |server, imap|
+      server.on("FETCH") do |resp|
+        resp.untagged("123 FETCH (UID 1111 FLAGS (\\Seen $MDNSent))")
+        resp.untagged("456 FETCH (UID 4444 FLAGS (\\Seen \\Answered))")
+        resp.untagged("789 FETCH (UID 7777 FLAGS ())")
+        resp.done_ok
+      end
+      fetched = imap.fetch [123, 456, 789], %w[UID FLAGS]
+      assert_equal 3, fetched.size
+      assert_instance_of Net::IMAP::FetchData, fetched[0]
+      assert_instance_of Net::IMAP::FetchData, fetched[1]
+      assert_instance_of Net::IMAP::FetchData, fetched[2]
+      assert_equal 123,  fetched[0].seqno
+      assert_equal 456,  fetched[1].seqno
+      assert_equal 789,  fetched[2].seqno
+      assert_equal 1111, fetched[0].uid
+      assert_equal 4444, fetched[1].uid
+      assert_equal 7777, fetched[2].uid
+      assert_equal [:Seen, "$MDNSent"], fetched[0].flags
+      assert_equal [:Seen, :Answered],  fetched[1].flags
+      assert_equal [],                  fetched[2].flags
+      assert_equal("RUBY0002 FETCH 123,456,789 (UID FLAGS)",
+                   server.commands.pop.raw.strip)
+    end
+  end
+
+  test "#uid_fetch with UIDFETCH responses" do
+    with_fake_server select: "inbox" do |server, imap|
+      server.on("UID FETCH") do |resp|
+        resp.untagged("1111 UIDFETCH (FLAGS (\\Seen $MDNSent))")
+        resp.untagged("4444 UIDFETCH (FLAGS (\\Seen \\Answered))")
+        resp.untagged("7777 UIDFETCH (FLAGS ())")
+        resp.done_ok
+      end
+      fetched = imap.uid_fetch [123, 456, 789], %w[FLAGS]
+      assert_equal 3, fetched.size
+      assert_instance_of Net::IMAP::UIDFetchData, fetched[0]
+      assert_instance_of Net::IMAP::UIDFetchData, fetched[1]
+      assert_instance_of Net::IMAP::UIDFetchData, fetched[2]
+      assert_equal 1111, fetched[0].uid
+      assert_equal 4444, fetched[1].uid
+      assert_equal 7777, fetched[2].uid
+      assert_equal [:Seen, "$MDNSent"], fetched[0].flags
+      assert_equal [:Seen, :Answered],  fetched[1].flags
+      assert_equal [],                  fetched[2].flags
+      assert_equal("RUBY0002 UID FETCH 123,456,789 (FLAGS)",
+                   server.commands.pop.raw.strip)
+    end
+  end
+
   test "#fetch with changedsince" do
     with_fake_server select: "inbox" do |server, imap|
       server.on("FETCH", &:done_ok)
-      imap.fetch 1..-1, %w[FLAGS], changedsince: 12345
+      fetched = imap.fetch 1..-1, %w[FLAGS], changedsince: 12345
+      assert_empty fetched
       assert_equal("RUBY0002 FETCH 1:* (FLAGS) (CHANGEDSINCE 12345)",
                    server.commands.pop.raw.strip)
     end
@@ -1134,12 +1267,85 @@ EOF
   test "#uid_fetch with changedsince" do
     with_fake_server select: "inbox" do |server, imap|
       server.on("UID FETCH", &:done_ok)
-      imap.uid_fetch 1..-1, %w[FLAGS], changedsince: 12345
+      fetched = imap.uid_fetch 1..-1, %w[FLAGS], changedsince: 12345
+      assert_empty fetched
       assert_equal("RUBY0002 UID FETCH 1:* (FLAGS) (CHANGEDSINCE 12345)",
                    server.commands.pop.raw.strip)
     end
   end
 
+  test "#uid_fetch with partial" do
+    with_fake_server select: "inbox" do |server, imap|
+      server.on("UID FETCH", &:done_ok)
+      imap.uid_fetch 1.., "FAST", partial: 1..500
+      assert_equal("RUBY0002 UID FETCH 1:* FAST (PARTIAL 1:500)",
+                   server.commands.pop.raw.strip)
+      imap.uid_fetch 1.., "FAST", partial: 1...501
+      assert_equal("RUBY0003 UID FETCH 1:* FAST (PARTIAL 1:500)",
+                   server.commands.pop.raw.strip)
+      imap.uid_fetch 1.., "FAST", partial: -500..-1
+      assert_equal("RUBY0004 UID FETCH 1:* FAST (PARTIAL -500:-1)",
+                   server.commands.pop.raw.strip)
+      imap.uid_fetch 1.., "FAST", partial: -500...-1
+      assert_equal("RUBY0005 UID FETCH 1:* FAST (PARTIAL -500:-2)",
+                   server.commands.pop.raw.strip)
+      imap.uid_fetch 1.., "FAST", partial: 1..20, changedsince: 1234
+      assert_equal("RUBY0006 UID FETCH 1:* FAST (PARTIAL 1:20 CHANGEDSINCE 1234)",
+                   server.commands.pop.raw.strip)
+    end
+  end
+
+  test "#store with FETCH responses" do
+    with_fake_server select: "inbox" do |server, imap|
+      server.on("STORE") do |resp|
+        resp.untagged("123 FETCH (UID 1111 FLAGS (\\Seen $MDNSent))")
+        resp.untagged("456 FETCH (UID 4444 FLAGS (\\Seen \\Answered))")
+        resp.untagged("789 FETCH (UID 7777 FLAGS (\\Seen))")
+        resp.done_ok
+      end
+      changed = imap.store [123, 456, 789], "+FLAGS", %i[Seen]
+      assert_equal("RUBY0002 STORE 123,456,789 +FLAGS (\\Seen)",
+                   server.commands.pop.raw.strip)
+      assert_equal 3, changed.size
+      assert_instance_of Net::IMAP::FetchData, changed[0]
+      assert_instance_of Net::IMAP::FetchData, changed[1]
+      assert_instance_of Net::IMAP::FetchData, changed[2]
+      assert_equal 123,  changed[0].seqno
+      assert_equal 456,  changed[1].seqno
+      assert_equal 789,  changed[2].seqno
+      assert_equal 1111, changed[0].uid
+      assert_equal 4444, changed[1].uid
+      assert_equal 7777, changed[2].uid
+      assert_equal [:Seen, "$MDNSent"], changed[0].flags
+      assert_equal [:Seen, :Answered],  changed[1].flags
+      assert_equal [:Seen],             changed[2].flags
+    end
+  end
+
+  test "#uid_store with UIDFETCH responses" do
+    with_fake_server select: "inbox" do |server, imap|
+      server.on("UID STORE") do |resp|
+        resp.untagged("1111 UIDFETCH (FLAGS (\\Seen $MDNSent))")
+        resp.untagged("4444 UIDFETCH (FLAGS (\\Seen \\Answered))")
+        resp.untagged("7777 UIDFETCH (FLAGS (\\Seen))")
+        resp.done_ok
+      end
+      changed = imap.uid_store [123, 456, 789], "+FLAGS", %i[Seen]
+      assert_equal("RUBY0002 UID STORE 123,456,789 +FLAGS (\\Seen)",
+                   server.commands.pop.raw.strip)
+      assert_equal 3, changed.size
+      assert_instance_of Net::IMAP::UIDFetchData, changed[0]
+      assert_instance_of Net::IMAP::UIDFetchData, changed[1]
+      assert_instance_of Net::IMAP::UIDFetchData, changed[2]
+      assert_equal 1111, changed[0].uid
+      assert_equal 4444, changed[1].uid
+      assert_equal 7777, changed[2].uid
+      assert_equal [:Seen, "$MDNSent"], changed[0].flags
+      assert_equal [:Seen, :Answered],  changed[1].flags
+      assert_equal [:Seen],             changed[2].flags
+    end
+  end
+
   test "#store with unchangedsince" do
     with_fake_server select: "inbox" do |server, imap|
       server.on("STORE", &:done_ok)
@@ -1162,6 +1368,65 @@ EOF
     end
   end
 
+  test "#expunge with EXPUNGE responses" do
+    with_fake_server(select: "INBOX") do |server, imap|
+      server.on "EXPUNGE" do |resp|
+        resp.untagged("1 EXPUNGE")
+        resp.untagged("1 EXPUNGE")
+        resp.untagged("99 EXPUNGE")
+        resp.done_ok
+      end
+      response = imap.expunge
+      cmd = server.commands.pop
+      assert_equal ["EXPUNGE", nil], [cmd.name, cmd.args]
+      assert_equal [1, 1, 99], response
+      assert_equal [], imap.clear_responses("EXPUNGED")
+    end
+  end
+
+  test "#expunge with a VANISHED response" do
+    with_fake_server(select: "INBOX") do |server, imap|
+      server.on "EXPUNGE" do |resp|
+        resp.untagged("VANISHED 15:456")
+        resp.done_ok
+      end
+      response = imap.expunge
+      cmd = server.commands.pop
+      assert_equal ["EXPUNGE", nil], [cmd.name, cmd.args]
+      assert_equal(
+        Net::IMAP::VanishedData[uids: [15..456], earlier: false],
+        response
+      )
+      assert_equal([], imap.clear_responses("VANISHED"))
+    end
+  end
+
+  test "#expunge with multiple VANISHED responses" do
+    with_fake_server(select: "INBOX") do |server, imap|
+      server.unsolicited("VANISHED 86")
+      server.on "EXPUNGE" do |resp|
+        resp.untagged("VANISHED (EARLIER) 1:5,99,123")
+        resp.untagged("VANISHED 15,456")
+        resp.untagged("VANISHED (EARLIER) 987,1001")
+        resp.done_ok
+      end
+      response = imap.expunge
+      cmd = server.commands.pop
+      assert_equal ["EXPUNGE", nil], [cmd.name, cmd.args]
+      assert_equal(
+        Net::IMAP::VanishedData[uids: [15, 86, 456], earlier: false],
+        response
+      )
+      assert_equal(
+        [
+          Net::IMAP::VanishedData[uids: [1..5, 99, 123], earlier: true],
+          Net::IMAP::VanishedData[uids: [987, 1001],     earlier: true],
+        ],
+        imap.clear_responses("VANISHED")
+      )
+    end
+  end
+
   def test_close
     with_fake_server(select: "inbox") do |server, imap|
       resp = imap.close
@@ -1202,16 +1467,175 @@ EOF
       end
 
       server.on "SEARCH", &search_resp
-      assert_equal search_result, imap.search(["subject", "hello",
+      assert_equal search_result, imap.search(["subject", "hello world",
                                                [1..5, 8, 10..-1]])
       cmd = server.commands.pop
-      assert_equal ["SEARCH", "subject hello 1:5,8,10:*"], [cmd.name, cmd.args]
+      assert_equal(
+        ["SEARCH",'subject "hello world" 1:5,8,10:*'],
+        [cmd.name, cmd.args]
+      )
+
+      imap.search(["OR", 1..1000, -1, "UID", 12345..-1])
+      assert_equal "OR 1:1000 * UID 12345:*", server.commands.pop.args
+
+      imap.search([1..1000, "UID", 12345..])
+      assert_equal "1:1000 UID 12345:*", server.commands.pop.args
+
+      # Unfortunately, we can't send every sequence-set string directly
+      imap.search(["SUBJECT", "1,*"])
+      assert_equal 'SUBJECT "1,*"', server.commands.pop.args
+
+      imap.search(["subject", "hello", Set[1, 2, 3, 4, 5, 8, *(10..100)]])
+      assert_equal "subject hello 1:5,8,10:100", server.commands.pop.args
+
+      imap.search('SUBJECT "Hello world"', "UTF-8")
+      assert_equal 'CHARSET UTF-8 SUBJECT "Hello world"', server.commands.pop.args
+
+      imap.search('CHARSET UTF-8 SUBJECT "Hello world"')
+      assert_equal 'CHARSET UTF-8 SUBJECT "Hello world"', server.commands.pop.args
+
+      imap.search('SUBJECT "Hello world"', charset: "UTF-8")
+      assert_equal 'CHARSET UTF-8 SUBJECT "Hello world"', server.commands.pop.args
+
+      imap.search([:*])
+      assert_equal "*", server.commands.pop.args
+
+      seqset_coercible = Object.new
+      def seqset_coercible.to_sequence_set
+        Net::IMAP::SequenceSet[1..9]
+      end
+      imap.search([seqset_coercible])
+      assert_equal "1:9", server.commands.pop.args
 
       server.on "UID SEARCH", &search_resp
       assert_equal search_result, imap.uid_search(["subject", "hello",
                                                    [1..22, 30..-1]])
       cmd = server.commands.pop
       assert_equal ["UID SEARCH", "subject hello 1:22,30:*"], [cmd.name, cmd.args]
+
+      assert_equal search_result, imap.search(
+        "RETURN (COUNT) NOT (FLAGGED (OR SEEN ANSWERED))"
+      )
+      cmd = server.commands.pop
+      assert_equal "RETURN (COUNT) NOT (FLAGGED (OR SEEN ANSWERED))", cmd.args
+
+      assert_equal search_result, imap.search([
+        "RETURN", %w(MIN MAX COUNT), "NOT", ["FLAGGED", %w(OR SEEN ANSWERED)]
+      ])
+      cmd = server.commands.pop
+      assert_equal "RETURN (MIN MAX COUNT) NOT (FLAGGED (OR SEEN ANSWERED))", cmd.args
+
+      assert_equal search_result, imap.search(
+        ["NOT", ["FLAGGED", %w(OR SEEN ANSWERED)]], return: %w(MIN MAX COUNT)
+      )
+      cmd = server.commands.pop
+      assert_equal "RETURN (MIN MAX COUNT) NOT (FLAGGED (OR SEEN ANSWERED))", cmd.args
+
+      assert_equal search_result, imap.search(
+        ["UID", 1234..], return: %w(PARTIAL -500:-1)
+      )
+      cmd = server.commands.pop
+      assert_equal "RETURN (PARTIAL -500:-1) UID 1234:*", cmd.args
+
+      assert_equal search_result, imap.search(
+        ["UID", 1234..], return: [:PARTIAL, "-500:-1"]
+      )
+      cmd = server.commands.pop
+      assert_equal "RETURN (PARTIAL -500:-1) UID 1234:*", cmd.args
+
+      assert_equal search_result, imap.search(
+        ["UID", 1234..], return: [:PARTIAL, -500..-1, :FOO, 1..]
+      )
+      cmd = server.commands.pop
+      assert_equal "RETURN (PARTIAL -500:-1 FOO 1:*) UID 1234:*", cmd.args
+    end
+  end
+
+  test("#search/#uid_search with invalid arguments") do
+    with_fake_server do |server, imap|
+      server.on "SEARCH"     do |cmd| cmd.fail_no "should fail before this" end
+      server.on "UID SEARCH" do |cmd| cmd.fail_no "should fail before this" end
+
+      assert_raise(ArgumentError) do
+        imap.search(["charset", "foo", "ALL"], "bar")
+      end
+      assert_raise(ArgumentError) do
+        imap.search("charset foo ALL", "bar")
+      end
+      assert_raise(ArgumentError) do
+        imap.search(["ALL"], "foo", charset: "bar")
+      end
+      assert_raise(ArgumentError) do
+        imap.search(["charset", "foo", "ALL"], charset: "bar")
+      end
+      assert_raise(ArgumentError) do
+        imap.search("charset foo ALL", charset: "bar")
+      end
+      # Parsing return opts is too complicated, for now.
+      # assert_raise(ArgumentError) do
+      #   imap.search("return () charset foo ALL", "bar")
+      # end
+
+      assert_raise(ArgumentError) do
+        imap.search(["retURN", %w(foo bar), "ALL"], return: %w[foo bar])
+      end
+      assert_raise(ArgumentError) do
+        imap.search("RETURN (foo bar) ALL", return: %w[foo bar])
+      end
+      assert_raise(TypeError) do
+        imap.search("ALL", return: "foo bar")
+      end
+      assert_raise(TypeError) do
+        imap.search(["retURN", "foo bar", "ALL"])
+      end
+    end
+  end
+
+  test("#search/#uid_search with ESEARCH or IMAP4rev2") do
+    with_fake_server do |server, imap|
+      # Example from RFC9051, 6.4.4:
+      #   C: A282 SEARCH RETURN (MIN COUNT) FLAGGED
+      #       SINCE 1-Feb-1994 NOT FROM "Smith"
+      #   S: * ESEARCH (TAG "A282") MIN 2 COUNT 3
+      #   S: A282 OK SEARCH completed
+      server.on "SEARCH" do |cmd|
+        cmd.untagged "ESEARCH", "(TAG \"unrelated1\") MIN 1 COUNT 2"
+        cmd.untagged "ESEARCH", "(TAG %p) MIN 2 COUNT 3" % [cmd.tag]
+        cmd.untagged "ESEARCH", "(TAG \"unrelated2\") MIN 222 COUNT 333"
+        cmd.done_ok
+      end
+      result = imap.search(
+        'RETURN (MIN COUNT) FLAGGED SINCE 1-Feb-1994 NOT FROM "Smith"'
+      )
+      cmd = server.commands.pop
+      assert_equal Net::IMAP::ESearchResult.new(
+        cmd.tag, false, [["MIN", 2], ["COUNT", 3]]
+      ), result
+      esearch_responses = imap.clear_responses("ESEARCH")
+      assert_equal 2, esearch_responses.count
+      refute esearch_responses.include?(result)
+    end
+  end
+
+  test("missing server ESEARCH response") do
+    with_fake_server do |server, imap|
+      # Example from RFC9051, 6.4.4:
+      #   C: A282 SEARCH RETURN (SAVE) FLAGGED SINCE 1-Feb-1994 NOT FROM "Smith"
+      #   S: A282 OK SEARCH completed, result saved
+      server.on "SEARCH"     do |cmd| cmd.done_ok "result saved" end
+      server.on "UID SEARCH" do |cmd| cmd.done_ok "result saved" end
+      result = imap.search(
+        'RETURN (SAVE) FLAGGED SINCE 1-Feb-1994 NOT FROM "Smith"'
+      )
+      assert_pattern do
+        result => Net::IMAP::ESearchResult[uid: false, tag: /^RUBY\d+/, data: []]
+      end
+      result = imap.uid_search(
+        'RETURN (SAVE) FLAGGED SINCE 1-Feb-1994 NOT FROM "Smith"'
+      )
+      assert_pattern do
+        result => Net::IMAP::ESearchResult[uid: true, tag: /^RUBY\d+/, data: []]
+      end
     end
   end
 
@@ -1220,9 +1644,11 @@ EOF
       server.on "SEARCH",     &:done_ok
       server.on "UID SEARCH", &:done_ok
       found = imap.search ["subject", "hello"]
-      assert_equal [], found
+      assert_instance_of Net::IMAP::SearchResult, found
+      assert_empty found
       found = imap.uid_search ["subject", "hello"]
-      assert_equal [], found
+      assert_instance_of Net::IMAP::SearchResult, found
+      assert_empty found
     end
   end
 
